import argparse
import json
import os
import sys

from .app import helm_upgrade

HERE = os.path.abspath(os.path.dirname(__file__))


def parse_args(args):
    """Parse arguments from the command line"""
    DESCRIPTION = (
        "Update the dependencies of a local Helm Chart in a project repository."
    )
    parser = argparse.ArgumentParser(description=DESCRIPTION)

    parser.add_argument(
        "chart_path",
        type=str,
        help="Path to the file containing the dependencies of the local Helm Chart to be updated.",
    )

    parser.add_argument(
        "dependencies",
        type=json.loads,
        help="""A dictionary of Helm Chart dependencies and their host repo URLs.
        E.g. '{"nginx-ingress":
        "https://raw.githubusercontent.com/helm/charts/master/stable/nginx-ingress/Chart.yaml"}'
        """,
    )

    parser.add_argument(
        "--dry-run",
        action="store_true",
        help="Perform a dry run of the update. Don't write the changes to a file.",
    )

    return parser.parse_args()


def main():
    """Main function"""
    args = parse_args(sys.argv[1:])

    helm_upgrade(
        args.chart_path,
        args.dependencies,
        dry_run=args.dry_run,
    )


if __name__ == "__main__":
    main()



import os
import re
from itertools import compress

import numpy as np
import requests
from bs4 import BeautifulSoup
from ruamel.yaml import YAML

HERE = os.path.abspath(os.getcwd())
yaml = YAML(typ="safe", pure=True)


def get_request(url: str):
    """Send a HTTP GET request to a target URL. Return payload as JSON.

    Args:
        url (str): The URL to send the request to
    """
    resp = requests.get(url)

    if not resp:
        raise Exception(f"Response not returned by URL: {url}")

    return resp.text


def update_requirements_file(
    chart_path: str,
    deps_to_update: list,
    deps: dict,
):
    """Update the local Helm Chart with new dependency versions

    Args:
        chart_path (str): Path to the file containing the dependencies of the
            local helm chart
        deps_to_update (list): The dependencies to be updated
        deps (dict): The dependencies and their versions
    """
    file_path = os.path.join(HERE, chart_path)

    with open(file_path, "r") as stream:
        chart_yaml = yaml.load(stream)

    for dep in deps_to_update:
        for dependency in chart_yaml["dependencies"]:
            if dependency["name"] == dep:
                dependency["version"] = deps[dep]

    with open(file_path, "w") as stream:
        yaml.dump(chart_yaml, stream)


def check_chart_versions(
    current_deps: dict,
    new_deps: dict,
) -> list:
    """Check whether the versions of the charts in the current dependencies are
    up-to-date with the remote ones.

    Args:
        current_deps (dict): The versions the helm chart is currently running
        new_deps (dict): Newer versions of the dependencies

    Returns:
        list: A list of the dependencies that need updating
    """
    charts = list(current_deps.keys())
    condition = [(current_deps[chart] != new_deps[chart]) for chart in charts]

    if np.any(condition):
        return list(compress(charts, condition))
    else:
        return []


def pull_version_from_chart_file(output_dict: dict, dependency: str, url: str) -> dict:
    """Pull recent, up-to-date version from remote host stored in a Chart.yml
    file.

    Args:
        output_dict (dict): The dictionary to store versions in
        dependency (str): The dependency to get a new version for
        url (str): The URL of the remotely hosted versions
    """
    chart_reqs = yaml.load(get_request(url))
    output_dict[dependency] = chart_reqs["version"]

    return output_dict


def pull_version_from_github_pages(
    output_dict: dict, dependency: str, url: str
) -> dict:
    """Pull recent, up-to-date version from remote host listed on a GitHub Pages
    site.

    Args:
        output_dict (dict): The dictionary to store versions in
        dependency (str): The dependency to get a version for
        url (str): The URL of the remotely hosted versions
    """
    chart_reqs = yaml.load(get_request(url))
    updates_sorted = sorted(
        chart_reqs["entries"][dependency], key=lambda k: k["created"]
    )
    output_dict[dependency] = updates_sorted[-1]["version"]

    return output_dict


def pull_version_from_github_releases(
    output_dict: dict, dependency: str, url: str
) -> dict:
    """Pull recent, up-to-date version from remote host listed on a GitHub
    Releases site.

    Args:
        output_dict (dict): The dictionary to store versions in
        dependency (str): The dependency to get a version for
        url (str): The URL of the remotely hosted versions
    """
    res = get_request(url)
    soup = BeautifulSoup(res, "html.parser")
    titles = soup.find_all("title")

    for title in titles:
        match = re.search(r"v[0-9]*\.[0-9]*\.[0-9]*", title.text)
        output_dict[dependency] = match.group(0)

    return output_dict


def get_remote_chart_versions(deps: dict) -> dict:
    """Get dependency versions from the remote hosts

    Args:
        deps (dict): The dependencies to check and their host URLs
        dependency (str): The dependency to get a version for

    Returns:
        dict: The dependencies and the most recent versions from the remote
              hosts
    """
    remote_dependencies = {}

    for dep in deps.keys():
        if deps[dep].endswith("Chart.yaml"):
            remote_dependencies = pull_version_from_chart_file(
                remote_dependencies,
                dep,
                deps[dep],
            )

        elif "/gh-pages/" in deps[dep]:
            remote_dependencies = pull_version_from_github_pages(
                remote_dependencies,
                dep,
                deps[dep],
            )

        elif deps[dep].endswith("/releases/latest"):
            remote_dependencies = pull_version_from_github_releases(
                remote_dependencies,
                dep,
                deps[dep],
            )

        else:
            raise Exception(f"Chart type not recognised: {deps[dep]}")

    return remote_dependencies


def get_local_chart_versions(chart_path: str) -> dict:
    """The dependency versions currently being installed by the helm chart

    Args:
        chart_path (str): The path to the file containing the dependencies of
            the local helm chart
    """
    local_dependencies = {}

    filepath = os.path.join(HERE, chart_path)

    with open(filepath, "r") as stream:
        chart_deps = yaml.load(stream)

    for dependency in chart_deps["dependencies"]:
        local_dependencies[dependency["name"]] = dependency["version"]

    return local_dependencies


def helm_upgrade(
    chart_path: str,
    dependencies: dict,
    dry_run: bool = False,
):
    """Upgrade a local helm chart to have up-to-date versions of its
    dependencies.

    Args:
        chart_path (str): The path to the file that contains the dependencies of
            the local helm chart to be updated
        dependencies (dict): A list of dependencies to check and host URLs
        dry_run (bool, optional): Don't change any files. Defaults to False.
    """
    # Get local dependencies
    local_deps = get_local_chart_versions(chart_path)
    # Get remote dependencies
    remote_deps = get_remote_chart_versions(dependencies)
    # Check the chart versions
    charts_to_update = check_chart_versions(local_deps, remote_deps)

    if (len(charts_to_update) > 0) and (not dry_run):
        update_requirements_file(chart_path, charts_to_update, remote_deps)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['helm_upgrade']

package_data = \
{'': ['*']}

install_requires = \
['bs4==0.0.1',
 'numpy==1.22.3',
 'requests==2.27.1',
 'ruamel.yaml>=0.17.21,<0.18.0']

entry_points = \
{'console_scripts': ['helm-upgrade = helm_upgrade.cli:main']}

setup_kwargs = {
    'name': 'helm-upgrade',
    'version': '0.1.1',
    'description': 'A Python CLI to manage Helm Chart dependencies',
    'long_description': '# helm-upgrade\n\n[![PyPI version](https://badge.fury.io/py/helm-upgrade.svg)](https://badge.fury.io/py/helm-upgrade) [![pre-commit.ci status](https://results.pre-commit.ci/badge/github/sgibson91/helm-upgrade/main.svg)](https://results.pre-commit.ci/latest/github/sgibson91/helm-upgrade/main) [![CI Tests](https://github.com/sgibson91/helm-upgrade/actions/workflows/ci.yaml/badge.svg)](https://github.com/sgibson91/helm-upgrade/actions/workflows/ci.yaml) [![codecov](https://codecov.io/gh/sgibson91/helm-upgrade/branch/main/graph/badge.svg?token=U2HTE7X6BK)](https://codecov.io/gh/sgibson91/helm-upgrade)\n\nDo you manage a Helm Chart that has dependencies on other Helm Charts?\nAre you fed up of manually updating these dependencies?\nThen this is the tool for you!\n`helm-upgrade` is a Python command line interface (CLI) that automatically updates the dependencies of local Helm Charts.\n\nThis tool was inspired by [`bump-helm-deps-action`](https://github.com/sgibson91/bump-helm-deps-action).\n\n**Table of Contents**\n\n- [:rocket: Installation](#rocket-installation)\n  - [:snake: `pip`](#snake-pip)\n  - [:wrench: Manual](#wrench-manual)\n- [:recycle: Usage](#recycle-usage)\n  - [:wheel_of_dharma: Remote Helm Charts](#wheel_of_dharma-remote-helm-charts)\n- [:white_check_mark: Running Tests](#white_check_mark-running-tests)\n- [:sparkles: Contributing](#sparkles-contributing)\n\n---\n\n## :rocket: Installation\n\nIt\'s recommended to use Python version 3.8 with this tool.\n\n### :snake: `pip`\n\n```bash\npip install helm-upgrade\n```\n\n### :wrench: Manual\n\nFirst of all, clone this repository and change into it.\n\n```bash\ngit clone https://github.com/sgibson91/helm-upgrade.git\ncd helm-upgrade\n```\n\nUse Python to install requirements and the package.\nPython 3.8 is recommended.\n\n```bash\npython -m pip install .\n```\n\nTest the installation by calling the help page.\n\n```bash\nhelm-upgrade --help\n```\n\n## :recycle: Usage\n\n```\nusage: helm-upgrade [-h] [--dry-run] chart_path dependencies\n\nUpdate the dependencies of a local Helm Chart in a project repository.\n\npositional arguments:\n  chart_path    Path to the file containing the dependencies of the local Helm Chart to\n                be updated.\n  dependencies  A dictionary of Helm Chart dependencies and their host repo URLs. E.g.\n                \'{"nginx-ingress":\n                "https://raw.githubusercontent.com/helm/charts/master/stable/nginx-\n                ingress/Chart.yaml"}\'\n\noptional arguments:\n  -h, --help    show this help message and exit\n  --dry-run     Perform a dry run of the update. Don\'t write the changes to a file.\n```\n\n`helm-upgrade` will then:\n\n1) read the current versions of your dependencies from the file you specify,\n2) find the latest versions of your desired dependencies from the URLs provided (in JSON schema) to the `dependencies` argument,\n3) compare whether these versions are equal,\n4) if the versions are not equal (and the `--dry-run` flag has not been set), your helm chart dependencies will be overwritten with the new chart versions.\n\n### :wheel_of_dharma: Remote Helm Charts\n\n`helm-upgrade` currently recognises chart versions from three types of hosts.\n\n1) A `Chart.yaml` file from another GitHub repository.\n   These URLs end with "`/Chart.yaml`".\n\n   For example, [https://raw.githubusercontent.com/helm/charts/master/stable/nginx-ingress/Chart.yaml](https://github.com/helm/charts/blob/master/stable/nginx-ingress/Chart.yaml)\n\n2) A repository of chart versions hosted on GitHub pages.\n   These URLs contain "`/gh-pages/`".\n\n   For example, [https://raw.githubusercontent.com/jupyterhub/helm-chart/gh-pages/index.yaml](https://github.com/jupyterhub/helm-chart/blob/gh-pages/index.yaml)\n\n3) Versions listed on a GitHub Releases page.\n   These URLs end with "`/releases/latest`" and uses `BeautifulSoup` to search the html.\n\n   For example, <https://github.com/jetstack/cert-manager/releases/latest>\n\n## :white_check_mark: Running Tests\n\nTo run the test suite, you must first following the [manual installation instructions](#wrench-manual).\nOnce completed, the test suite can be run as follows:\n\n```bash\npython -m pytest -vvv\n```\n\nTo see code coverage of the test suite, run the following:\n\n```bash\npython -m coverage run -m pytest -vvv\ncoverage report\n```\n\nAn interactive HTML version of the report can be accessed by running the following:\n\n```bash\ncoverage html\n```\n\nAnd then opening the `htmlcov/index.html` file in a browser window.\n\n## :sparkles: Contributing\n\n:tada: Thank you for wanting to contribute! :tada:\nMake sure to read our [Code of Conduct](CODE_OF_CONDUCT.md) and [Contributing Guidelines](CONTRIBUTING.md) to get you started.\n',
    'author': 'Sarah Gibson',
    'author_email': 'drsarahlgibson@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
